package dev.fyloz.colorrecipesexplorer.rest.files

import dev.fyloz.colorrecipesexplorer.config.properties.CreProperties
import dev.fyloz.colorrecipesexplorer.rest.noContent
import dev.fyloz.colorrecipesexplorer.service.files.FileService
import org.springframework.core.io.ByteArrayResource
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile
import java.net.URI

const val FILE_CONTROLLER_PATH = "/api/file"
private const val DEFAULT_MEDIA_TYPE = MediaType.APPLICATION_OCTET_STREAM_VALUE

@RestController
@RequestMapping(FILE_CONTROLLER_PATH)
class FileController(
    private val fileService: FileService,
    private val creProperties: CreProperties
) {
    @GetMapping(produces = [MediaType.APPLICATION_OCTET_STREAM_VALUE])
    @PreAuthorize("hasAnyAuthority('READ_FILE')")
    fun upload(
        @RequestParam path: String,
        @RequestParam(required = false) mediaType: String?
    ): ResponseEntity<ByteArrayResource> {
        val file = fileService.read(path)
        return ResponseEntity.ok()
            .header("Content-Disposition", "filename=${getFileNameFromPath(path)}")
            .contentLength(file.contentLength())
            .contentType(MediaType.parseMediaType(mediaType ?: DEFAULT_MEDIA_TYPE))
            .body(file)
    }

    @PutMapping(consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorize("hasAnyAuthority('WRITE_FILE')")
    fun download(
        file: MultipartFile,
        @RequestParam path: String,
        @RequestParam(required = false) overwrite: Boolean = false
    ): ResponseEntity<Void> {
        fileService.write(file, path, overwrite)
        return created(path)
    }

    @DeleteMapping
    @PreAuthorize("hasAnyAuthority('REMOVE_FILE')")
    fun delete(@RequestParam path: String): ResponseEntity<Void> {
        return noContent {
            fileService.delete(path)
        }
    }

    private fun created(path: String): ResponseEntity<Void> =
        ResponseEntity
            .created(URI.create("${creProperties.deploymentUrl}$FILE_CONTROLLER_PATH?path=$path"))
            .build()

    private fun getFileNameFromPath(path: String) =
        path.split("/").last()
}
